@@ -30,6 +30,7 @@
 #include <libgnome/gnome-desktop-item.h>
 #include <libgnomeui/gnome-help.h>
 #include <libgnomeui/gnome-window-icon.h>
+#include <menu-tree.h>
 #include "dlg-pick-applications.h"
 #include "dlg-properties.h"
 #include "gtk-utils.h"
@@ -174,165 +175,94 @@ ok_cb (GtkWidget  *widget,
 
 
 void
-add_applications (DialogData  *data,
-		  GtkTreeIter *tree_iter,
-		  const char  *base_uri)
+add_applications_from_dir (DialogData         *data,
+			   GtkTreeIter        *tree_iter,
+			   MenuTreeDirectory  *menu_dir)
 {
-	QuickLounge             *quick_lounge = data->quick_lounge;
-	GtkTreeIter              iter;
-	GnomeVFSDirectoryHandle *handle;
-	GnomeVFSResult           result;
-	GnomeVFSFileInfo        *file_info;
+	QuickLounge *quick_lounge = data->quick_lounge;
+	GSList      *entries, *subdirs, *scan;
+	GtkTreeIter  iter;
 
-	result = gnome_vfs_directory_open (&handle, base_uri, (GNOME_VFS_FILE_INFO_DEFAULT | GNOME_VFS_FILE_INFO_FOLLOW_LINKS));
+	subdirs = menu_tree_directory_get_subdirs (menu_dir);
+	for (scan = subdirs; scan; scan = scan->next) {
+		MenuTreeDirectory *dir = scan->data;
+		const char        *icon_path = NULL;
+		GdkPixbuf         *icon = NULL;
 
-	if (result != GNOME_VFS_OK) {
-#ifdef DEBUG
-		g_print ("ERROR\n");
-#endif
-		return;
+		icon_path = menu_tree_directory_get_icon (dir);
+		if (icon_path != NULL) 
+			icon = create_pixbuf (quick_lounge->icon_theme, icon_path, ICON_SIZE_MENU);
+
+		gtk_tree_store_append (GTK_TREE_STORE (data->model), &iter, tree_iter);
+		gtk_tree_store_set (GTK_TREE_STORE (data->model), &iter,
+				    APP_URI_COLUMN, NULL,
+				    USE_APP_COLUMN, FALSE,
+				    IS_LAUNCHER_COLUMN, FALSE,
+				    APP_ICON_COLUMN, icon,
+				    APP_NAME_COLUMN, menu_tree_directory_get_name (dir),
+				    APP_COMMENT_COLUMN, menu_tree_directory_get_comment (dir),
+				    -1);
+
+		if (icon != NULL)
+			g_object_unref (icon);
+
+		add_applications_from_dir (data, &iter, dir);
+
+		menu_tree_directory_unref (dir);
 	}
+	g_slist_free (subdirs);
 
-	while (gtk_events_pending())
-		gtk_main_iteration();
+	entries = menu_tree_directory_get_entries (menu_dir);
+	for (scan = entries; scan; scan = scan->next) {
+		MenuTreeEntry *entry = scan->data;
+		const char    *icon_path = NULL;
+		GdkPixbuf     *icon = NULL;
 
-	file_info = gnome_vfs_file_info_new ();
-	while (gnome_vfs_directory_read_next (handle, file_info) == GNOME_VFS_OK) {
+		icon_path = menu_tree_entry_get_icon (entry);
+		if (icon_path != NULL)
+			icon = create_pixbuf (quick_lounge->icon_theme, icon_path, ICON_SIZE_MENU);
 
-		if (file_info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
-			GnomeDesktopItem *ditem = NULL;
-			GdkPixbuf        *icon = NULL;
-			char             *dot_desktop_uri;
-			char             *sub_menu_uri;
-			const char       *text = NULL;
-			char             *icon_path = NULL;
+		gtk_tree_store_append (GTK_TREE_STORE (data->model), &iter, tree_iter);
+		gtk_tree_store_set (GTK_TREE_STORE (data->model), &iter,
+				    APP_URI_COLUMN, menu_tree_entry_get_desktop_file_path (entry),
+				    USE_APP_COLUMN, FALSE,
+				    IS_LAUNCHER_COLUMN, TRUE,
+				    APP_ICON_COLUMN, icon,
+				    APP_NAME_COLUMN, menu_tree_entry_get_name (entry),
+				    APP_COMMENT_COLUMN, menu_tree_entry_get_comment (entry),
+				    -1);
 
-			sub_menu_uri = g_build_filename (base_uri, 
-							 file_info->name, 
-							 NULL);
-			dot_desktop_uri = g_build_filename (sub_menu_uri, 
-							    ".directory", 
-							    NULL);
+		if (icon != NULL)
+			g_object_unref (icon);
 
-			ditem = g_hash_table_lookup (quick_lounge->ditem_hash,
-						     dot_desktop_uri);
-
-			if (ditem == NULL) {
-
-				ditem = gnome_desktop_item_new_from_uri (dot_desktop_uri, GNOME_DESKTOP_ITEM_LOAD_ONLY_IF_EXISTS, NULL);
-
-				if (ditem == NULL) {
-					g_free (sub_menu_uri);
-					g_free (dot_desktop_uri);
-					continue;
-				}
-
-				gnome_desktop_item_ref (ditem);
-				g_hash_table_insert (quick_lounge->ditem_hash,
-						     g_strdup (dot_desktop_uri),
-						     ditem);
-			} else 
-				gnome_desktop_item_ref (ditem);
-
-			g_free (dot_desktop_uri);
-
-			text = gnome_desktop_item_get_localestring (ditem, "Name"); 
-
-			/**/
-
-			icon_path = gnome_desktop_item_get_icon (ditem, quick_lounge->icon_theme);
-			if (icon_path != NULL)
-				icon = create_pixbuf (icon_path, ICON_SIZE_MENU);
-
-			/**/
-
-			gtk_tree_store_append (GTK_TREE_STORE (data->model), &iter, tree_iter);
-			gtk_tree_store_set (GTK_TREE_STORE (data->model), &iter,
-					    APP_URI_COLUMN, sub_menu_uri,
-					    USE_APP_COLUMN, FALSE,
-					    IS_LAUNCHER_COLUMN, FALSE,
-					    APP_NAME_COLUMN, text,
-					    APP_ICON_COLUMN, icon,
-					    -1);
-
-			if (icon != NULL)
-				g_object_unref (icon);
-
-			add_applications (data, &iter, sub_menu_uri);
-			g_free (sub_menu_uri);
-
-			gnome_desktop_item_unref (ditem);
-
-		} else if (strcmp (file_info->name, ".directory") == 0) {
-			/* FIXME */
-
-		} else {
-			GnomeDesktopItem *ditem = NULL;
-			GdkPixbuf        *icon = NULL;
-			char             *desktop_uri;
-			const char       *text, *comment;
-			char             *icon_path = NULL;
-
-			desktop_uri = g_build_filename (base_uri, 
-							file_info->name, 
-							NULL);
-
-			ditem = g_hash_table_lookup (quick_lounge->ditem_hash, 
-						     desktop_uri);
-
-			if (ditem == NULL) {
-				ditem = gnome_desktop_item_new_from_uri (desktop_uri, GNOME_DESKTOP_ITEM_LOAD_ONLY_IF_EXISTS, NULL);
-
-				if (ditem == NULL) {
-					g_free (desktop_uri);
-					continue;
-				}
-
-				gnome_desktop_item_ref (ditem);
-				g_hash_table_insert (quick_lounge->ditem_hash,
-						     g_strdup (desktop_uri),
-						     ditem);
-			} else 
-				gnome_desktop_item_ref (ditem);			
-
-			text = gnome_desktop_item_get_localestring (ditem, "Name"); 
-			comment = gnome_desktop_item_get_localestring (ditem, "Comment"); 
-
-			/**/
-
-			icon_path = gnome_desktop_item_get_icon (ditem, quick_lounge->icon_theme);
-			if (icon_path != NULL)
-				icon = create_pixbuf (icon_path, ICON_SIZE_MENU);
-
-			/**/
-
-			gtk_tree_store_append (GTK_TREE_STORE (data->model), &iter, tree_iter);
-			gtk_tree_store_set (GTK_TREE_STORE (data->model), &iter,
-					    APP_URI_COLUMN, desktop_uri,
-					    USE_APP_COLUMN, FALSE,
-					    IS_LAUNCHER_COLUMN, TRUE,
-					    APP_ICON_COLUMN, icon,
-					    APP_NAME_COLUMN, text,
-					    APP_COMMENT_COLUMN, comment,
-					    -1);
-
-			if (icon != NULL)
-				g_object_unref (icon);
-
-			g_free (desktop_uri);
-
-			gnome_desktop_item_unref (ditem);
-		}
+		menu_tree_entry_unref (entry);
 	}
-
-	gnome_vfs_file_info_unref (file_info);
-	gnome_vfs_directory_close (handle);
-
-	if (tree_iter == NULL)
-		gtk_tree_view_expand_all (GTK_TREE_VIEW (data->tree_view));
+	g_slist_free (entries);
 }
 
 
+void
+add_applications (DialogData  *data,
+		  const char  *menu_file)
+{
+	MenuTree           *menu_tree;
+	MenuTreeDirectory  *menu_dir;
+
+	menu_tree = menu_tree_lookup (menu_file);
+	if (menu_tree == NULL)
+		return;
+
+	menu_dir = menu_tree_get_root_directory (menu_tree);
+	add_applications_from_dir (data, NULL, menu_dir);
+	menu_tree_directory_unref (menu_dir);
+
+	gtk_tree_view_expand_all (GTK_TREE_VIEW (data->tree_view));
+
+	menu_tree_unref (menu_tree);
+}
+
+
+
 static int
 name_column_sort_func (GtkTreeModel *model, 
                        GtkTreeIter  *a, 
@@ -554,5 +484,6 @@ dlg_pick_applications (QuickLounge  *quick_lounge,
 
 	gtk_widget_show (data->dialog);
 
-	add_applications (data, NULL, "applications://");
+	add_applications (data, "applications.menu"); 
+	add_applications (data, "settings.menu"); 
 }
