@@ -27,6 +27,7 @@
 
 #include <gtk/gtk.h>
 #include <libgnome/libgnome.h>
+#include <libgnomeui/gnome-help.h>
 #include <libgnomeui/gnome-ditem-edit.h>
 #include <libgnomeui/gnome-window-icon.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
@@ -43,7 +44,7 @@
 
 
 #define GLADE_FILE             "quick-lounge.glade"
-#define SEPARATOR              _("Space")
+#define SEPARATOR_NAME         _("Space")
 #define ICON_SIZE              24
 #define UPDATE_MAX_SIZE_DELAY  100
 
@@ -56,10 +57,23 @@ enum {
 };
 
 
-static GtkTargetEntry target_table[] = {
-	{ "text/uri-list", 0, 0 }
+enum {
+	TEXT_URI_LIST,
+	GTK_TREE_MODEL_ROW
 };
-static guint n_targets = sizeof (target_table) / sizeof (target_table[0]);
+
+static GtkTargetEntry source_target_table[] = {
+	{ "GTK_TREE_MODEL_ROW", GTK_TARGET_SAME_WIDGET, GTK_TREE_MODEL_ROW }
+};
+
+static guint source_target_table_n = G_N_ELEMENTS(source_target_table);
+
+static GtkTargetEntry dest_target_table[] = {
+	{ "text/uri-list", 0, TEXT_URI_LIST },
+	{ "GTK_TREE_MODEL_ROW", GTK_TARGET_SAME_WIDGET, GTK_TREE_MODEL_ROW }
+};
+
+static guint dest_target_table_n = G_N_ELEMENTS(dest_target_table);
 
 
 typedef struct {
@@ -90,14 +104,33 @@ typedef struct {
 	GtkTreeModel  *model;
 
 	int            timeout;
+	int            start_x, start_y;
 } DialogData;
 
 
+static gboolean
+change_max_size (gpointer user_data)
+{
+	DialogData *data = user_data;
+
+	g_source_remove (data->timeout);
+	data->timeout = 0;
+
+	quick_lounge_set_max_size (data->quick_lounge, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (data->p_maxsize_spinbutton)));
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->p_maxsize_spinbutton), quick_box_get_max_visible_cols (data->quick_box));
+
+	return FALSE;
+}
+
+
 /* called when the main dialog is closed. */
 static void
 destroy_cb (GtkWidget  *widget, 
 	    DialogData *data)
 {
+	if (data->timeout != 0) 
+		change_max_size (data);
+
 	g_object_unref (data->gui);
 	g_free (data);
 }
@@ -120,7 +153,13 @@ help_cb (GtkWidget  *widget,
 	GError *err;
 
 	err = NULL;  
-	gnome_help_display ("quick-lounge-applet", "properties", &err); 
+	gnome_help_display_desktop_on_screen (
+	      NULL, 
+	      "quick-lounge", 
+	      "quick-lounge", 
+	      "quick-lounge-prefs", 
+	      gtk_widget_get_screen (GTK_WIDGET (data->quick_lounge->applet)),
+	      &err); 
 	
 	if (err != NULL) {
 		GtkWidget *dialog;
@@ -388,7 +427,6 @@ update_list (DialogData *data)
 		GdkPixbuf   *pixbuf;
 		GtkTreeIter  iter;
 		double       scale;
-		char        *utf8_name;
 		int          w, h;
 
 		if (! QUICK_IS_BUTTON (child_info->widget))
@@ -396,7 +434,6 @@ update_list (DialogData *data)
 		button = QUICK_BUTTON (child_info->widget);
 
 		if (QUICK_IS_SEPARATOR (button)) { /* Space */
-			utf8_name = g_locale_to_utf8 (SEPARATOR, -1, NULL, NULL, NULL);
 			gtk_list_store_prepend (GTK_LIST_STORE (data->model), 
 						&iter);
 
@@ -406,9 +443,8 @@ update_list (DialogData *data)
 			gtk_list_store_set (GTK_LIST_STORE (data->model), 
 					    &iter,
 					    COLUMN_DATA, button,
-					    COLUMN_NAME, utf8_name,
+					    COLUMN_NAME, SEPARATOR_NAME,
 					    -1);
-			g_free (utf8_name);
 			continue;
 		}
 
@@ -528,7 +564,6 @@ add_space_cb (GtkWidget  *widget,
 	int               pos = -1;
 	gboolean          empty = FALSE;
 	GtkWidget        *button;
-	char             *utf8_name;
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (data->tree_view));
 	if (selection == NULL)
@@ -548,7 +583,6 @@ add_space_cb (GtkWidget  *widget,
 	
 	/* update the tree view */
 	
-	utf8_name = g_locale_to_utf8 (SEPARATOR, -1, NULL, NULL, NULL);
 	if (empty)
 		gtk_list_store_append (GTK_LIST_STORE (data->model), 
 				       &iter);
@@ -558,10 +592,8 @@ add_space_cb (GtkWidget  *widget,
 	gtk_list_store_set (GTK_LIST_STORE (data->model), 
 			    &iter,
 			    COLUMN_DATA, button,
-			    COLUMN_NAME, utf8_name,
+			    COLUMN_NAME, SEPARATOR_NAME,
 			    -1);
-	g_free (utf8_name);
-
 	data->n++;
 	
 	/* Save changes */
@@ -644,20 +676,7 @@ min_size_value_changed_cb (GtkWidget  *widget,
 			   DialogData *data)
 {
 	quick_lounge_set_min_visible_cols (data->quick_lounge, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (data->p_minsize_spinbutton)));
-}
-
-
-static gboolean
-change_max_size (gpointer user_data)
-{
-	DialogData *data = user_data;
-
-	g_source_remove (data->timeout);
-	data->timeout = 0;
-
-	quick_lounge_set_max_size (data->quick_lounge, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (data->p_maxsize_spinbutton)));
-
-	return FALSE;
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->p_minsize_spinbutton), quick_box_get_min_visible_cols (data->quick_box));
 }
 
 
@@ -736,11 +755,16 @@ drag_data_received  (GtkWidget          *widget,
 	int           pos;
 	GtkTreePath  *pos_path;
 	GtkTreeViewDropPosition drop_pos;
+	char         *selection_data = data->data;
+	GtkTreeIter   iter;
 
-        if (! ((data->length >= 0) && (data->format == 8))) {
-                gtk_drag_finish (context, FALSE, FALSE, time);
-                return;
-        }
+
+	if (!((data->length >= 0) && (data->format == 8))) {
+		gtk_drag_finish (context, FALSE, FALSE, time);
+		return;
+	}
+
+	gtk_drag_finish (context, TRUE, FALSE, time);
 
 	if (gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (ddata->tree_view),
 					       x, y,
@@ -751,53 +775,74 @@ drag_data_received  (GtkWidget          *widget,
 	} else
 		pos = 0;
 
-	list = get_file_list_from_url_list ((char *) data->data);
-	for (scan = list; scan; scan = scan->next) {
-		char      *src_uri = scan->data;
-		char      *uri;
-		GtkWidget *button;
+        /* separator 'uri' starts with a : as coded in drag_data_get() */
+	if (selection_data[0] == ':') { 
+		int          from_pos = atoi(selection_data + 1);
+		GtkWidget   *button;
 
-		if (! is_desktop_file (src_uri))
-			continue;
-
-		uri = g_strconcat (quick_lounge->location,
-				   "/",
-				   file_name_from_path (src_uri),
-				   NULL);
-
-		if (strcmp (src_uri, uri) == 0) {
-			button = get_button_from_uri (quick_box, uri);
-
-			if (button != NULL) {
-				int button_pos;
-				button_pos = get_child_position (quick_box, button);
-				if (button_pos != -1) 
-					quick_box_reorder_child (quick_box, button, pos);
-			}
-
-			g_free (uri);
-			continue;
-		}
-
-		g_free (uri);
-		uri = quick_lounge_util__get_unique_uri (quick_lounge);
-
-		if (! file_copy (src_uri, uri)) {
-			g_free (uri);
-			continue;
-		}
+		gtk_tree_model_iter_nth_child (ddata->model, &iter, NULL, from_pos);
 
 		if ((drop_pos == GTK_TREE_VIEW_DROP_BEFORE) ||
-		    (drop_pos == GTK_TREE_VIEW_DROP_INTO_OR_BEFORE))
-			pos--;
+		    (drop_pos == GTK_TREE_VIEW_DROP_INTO_OR_BEFORE)) {
+			if (from_pos < pos)
+				pos--;
+		}
+		
+		if ((drop_pos == GTK_TREE_VIEW_DROP_AFTER) ||
+		    (drop_pos == GTK_TREE_VIEW_DROP_INTO_OR_AFTER)) {
+			if (from_pos > pos)
+				pos++;
+		}
+		
+		button = tree_view_move_iter_to (ddata, &iter, pos, FALSE);
+		quick_box_reorder_child (ddata->quick_box, button, pos);
 
-		quick_box_add_button (quick_box, uri, ++pos);
+	} else {
+		list = get_file_list_from_url_list (selection_data);
+		for (scan = list; scan; scan = scan->next) {
+			char      *src_uri = scan->data;
+			char      *uri;
+			GtkWidget *button;
+
+			if (! is_desktop_file (src_uri))
+				continue;
+
+			uri = g_build_filename (quick_lounge->location,
+						file_name_from_path (src_uri),
+						NULL);
+			
+			if (strcmp (src_uri, uri) == 0) {
+				button = get_button_from_uri (quick_box, uri);
+				
+				if (button != NULL) {
+					int button_pos;
+					button_pos = get_child_position (quick_box, button);
+					if (button_pos != -1) 
+						quick_box_reorder_child (quick_box, button, pos);
+				}
+				
+				g_free (uri);
+				continue;
+			}
+			
+			g_free (uri);
+			uri = quick_lounge_util__get_unique_uri (quick_lounge);
+			
+			if (! file_copy (src_uri, uri)) {
+				g_free (uri);
+				continue;
+			}
+			
+			quick_box_add_button (quick_box, uri, ++pos);
+		}
+		path_list_free (list);
 	}
-	path_list_free (list);
 
-	gtk_drag_finish (context, TRUE, FALSE, time);
 	quick_lounge_save_order (quick_lounge);
 	dlg_properties_update (quick_lounge->prop_dialog);
+
+	gtk_tree_model_iter_nth_child (ddata->model, &iter, NULL, pos);
+	select_and_view_iter (ddata, &iter);
 }
 
 
@@ -812,19 +857,69 @@ drag_motion (GtkWidget          *widget,
 	DialogData              *ddata = extra_data;
 	GtkTreePath             *pos_path;
 	GtkTreeViewDropPosition  drop_pos;
+	gboolean                 result = FALSE;
 
 	if (! gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (ddata->tree_view),
 						 x, y,
 						 &pos_path,
 						 &drop_pos)) 
 		pos_path = gtk_tree_path_new_first ();
-	
-	gtk_tree_view_set_drag_dest_row  (GTK_TREE_VIEW (ddata->tree_view),
-					  pos_path,
-					  drop_pos);
+
+	if ((drop_pos != GTK_TREE_VIEW_DROP_INTO_OR_BEFORE)
+	    && (drop_pos != GTK_TREE_VIEW_DROP_INTO_OR_AFTER)) {
+		gtk_tree_view_set_drag_dest_row  (GTK_TREE_VIEW (ddata->tree_view),
+						  pos_path,
+						  drop_pos);
+		result = TRUE;
+	}
+
 	gtk_tree_path_free (pos_path);
 
-	return TRUE;
+	return result;
+}
+
+
+static gboolean
+tree_view_button_press_cb (GtkWidget      *widget,
+			   GdkEventButton *event,
+			   gpointer        data)
+{
+	DialogData *ddata = data;
+	ddata->start_x = event->x;
+	ddata->start_y = event->y;
+	return FALSE;
+}
+
+
+static void
+drag_data_begin (GtkWidget      *widget,
+		 GdkDragContext *context,
+		 gpointer        data)
+{
+	DialogData        *ddata = data;
+	GtkTreePath       *path = NULL;
+	GdkPixmap         *row_pix;
+	int                cell_x, cell_y;
+
+	gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (ddata->tree_view),
+				       ddata->start_x,
+				       ddata->start_y,
+				       &path,
+				       NULL,
+				       &cell_x,
+				       &cell_y);
+	
+	g_return_if_fail (path != NULL);
+
+	row_pix = gtk_tree_view_create_row_drag_icon (GTK_TREE_VIEW (ddata->tree_view), path);
+	gtk_drag_set_icon_pixmap (context,
+				  gdk_drawable_get_colormap (row_pix),
+				  row_pix,
+				  NULL,
+				  ddata->start_x + 1,
+				  cell_y + 1);
+	g_object_unref (row_pix);
+	gtk_tree_path_free (path);
 }
 
 
@@ -837,18 +932,21 @@ drag_data_get  (GtkWidget        *widget,
 		gpointer          data)
 {
 	DialogData       *ddata = data;
-        char             *target;
+        /*char             *target;*/
 	const char       *uri;
 	GtkTreeIter       iter;
 	GtkTreeSelection *selection;
 	GtkWidget        *button;
+	static char       pos_data[32];
 
+	/*
         target = gdk_atom_name (selection_data->target);
         if (strcmp (target, "text/uri-list") != 0) {
 		g_free (target);
 		return;
 	}
         g_free (target);
+	*/
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (ddata->tree_view));
 	gtk_tree_selection_get_selected (selection, NULL, &iter);
@@ -857,14 +955,23 @@ drag_data_get  (GtkWidget        *widget,
 			    COLUMN_DATA, &button,
 			    -1);
 	uri = g_object_get_data (G_OBJECT (button), "uri");
-	if (uri == NULL) 
-                return;
+	
+	if (uri == NULL) { /* separator */
+		int pos = get_iter_position (ddata, &iter);
 
-        gtk_selection_data_set (selection_data, 
-                                selection_data->target,
-                                8, 
-				uri,
-                                strlen (uri));
+		/* separators 'uri' are coded as a : and the separator position. */
+		sprintf (pos_data, ":%d", pos);
+		gtk_selection_data_set (selection_data, 
+					selection_data->target,
+					8, 
+					pos_data,
+					strlen(pos_data));
+	} else 
+		gtk_selection_data_set (selection_data, 
+					selection_data->target,
+					8, 
+					uri,
+					strlen (uri));
 }
 
 
@@ -880,8 +987,6 @@ dlg_properties (QuickLounge  *quick_lounge,
 	GtkWidget        *btn_help;
 	GtkTreeSelection *selection;
 
-	quick_lounge_set_textdomain ();
-
 	data = g_new0 (DialogData, 1);
 
 	data->quick_lounge = quick_lounge;
@@ -1008,28 +1113,41 @@ dlg_properties (QuickLounge  *quick_lounge,
 
 	/* Drag & Drop */
 
+	gtk_drag_source_set (data->tree_view,
+			     GDK_BUTTON1_MASK,
+			     source_target_table,
+			     source_target_table_n,
+			     GDK_ACTION_MOVE);
+
+	g_signal_connect (G_OBJECT (data->tree_view),
+			  "drag_begin",
+			  G_CALLBACK (drag_data_begin), 
+			  data);
+	g_signal_connect (G_OBJECT (data->tree_view),
+			  "drag_data_get",
+			  G_CALLBACK (drag_data_get), 
+			  data);
+
+	g_signal_connect (G_OBJECT (data->tree_view),
+			  "button_press_event",
+			  G_CALLBACK (tree_view_button_press_cb), 
+			  data);
+
 	gtk_drag_dest_set (GTK_WIDGET (data->tree_view),
                            GTK_DEST_DEFAULT_ALL,
-                           target_table, n_targets,
+                           dest_target_table,
+			   dest_target_table_n,
 			   GDK_ACTION_COPY | GDK_ACTION_MOVE);
-	
+
 	g_signal_connect (G_OBJECT (data->tree_view), 
 			  "drag_data_received",
 			  G_CALLBACK (drag_data_received), 
 			  data);
+
 	g_signal_connect (G_OBJECT (data->tree_view), 
 			  "drag_motion",
 			  G_CALLBACK (drag_motion), 
 			  data);
-	
-	gtk_drag_source_set (data->tree_view,
-			     GDK_BUTTON1_MASK,
-			     target_table, n_targets, 
-			     GDK_ACTION_COPY);
-	g_signal_connect (G_OBJECT (data->tree_view),
-			  "drag_data_get",
-			  G_CALLBACK (drag_data_get), 
-			  data);
 
 	/* run dialog. */
 
