@@ -1154,7 +1154,7 @@ arrow_button_pressed (GtkToggleButton *tbutton,
 
 		icon_path = gnome_desktop_item_get_icon (ditem, quick_box->priv->icon_theme);
 		if (icon_path != NULL)
-			icon = create_image (icon_path, ICON_SIZE_MENU);
+			icon = create_image (quick_box->priv->icon_theme, icon_path, ICON_SIZE_MENU);
 		if (icon != NULL)
 			gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), icon);
 
@@ -1325,7 +1325,7 @@ quick_box_init (QuickBox *quick_box)
 GType
 quick_box_get_type ()
 {
-        static guint type = 0;
+        static GType type = 0;
 
         if (! type) {
                 GTypeInfo type_info = {
