@@ -132,7 +132,7 @@ about_verb_cb (BonoboUIComponent *uic,
 	GdkPixbuf  *pixbuf = NULL;
         char       *file;
 	const char *authors[] = {
-		"Paolo Bacchilega <paolo.bacch@tin.it>",
+		"Paolo Bacchilega <paolo.bacchilega@libero.it>",
 		NULL
 	};
 	const char *documenters [] = {
@@ -150,9 +150,9 @@ about_verb_cb (BonoboUIComponent *uic,
         pixbuf = gdk_pixbuf_new_from_file (file, NULL);
         g_free(file);
 
-	quick_lounge->about_dialog = gnome_about_new (_("Quick Lounge"), 
+	quick_lounge->about_dialog = gnome_about_new (_("Launchers List"), 
 						      VERSION,
-						      "Copyright \xc2\xa9 2002-2003 Free Software Foundation, Inc.",
+						      "Copyright \xc2\xa9 2002-2004 Free Software Foundation, Inc.",
 						      _("Organize your favorite applications on the Panel"),
 						      authors,
 						      documenters,
@@ -197,7 +197,7 @@ remove_directory (const char *uri)
 	file_info = gnome_vfs_file_info_new ();
 	while (gnome_vfs_directory_read_next (h, file_info) == GNOME_VFS_OK) {
 		char *file_uri;
-		file_uri = g_strconcat (uri, "/", file_info->name, NULL);
+		file_uri = g_build_filename (uri, file_info->name, NULL);
 		gnome_vfs_unlink (file_uri);
 		g_free (file_uri);
 	}
@@ -272,21 +272,37 @@ static void
 applet_change_background (PanelApplet               *applet,
                           PanelAppletBackgroundType  type,
                           GdkColor                  *color,
-                          const gchar               *pixmap,
+                          GdkPixmap                 *pixmap,
                           QuickLounge               *quick_lounge)
 {
-	GtkWidget *widget = GTK_WIDGET (applet);
-
-	if (type == PANEL_NO_BACKGROUND) {
-		GtkRcStyle *rc_style = gtk_rc_style_new ();
-		gtk_widget_modify_style (widget, rc_style);
-		gtk_rc_style_unref (rc_style);
-
-	} else if (type == PANEL_COLOR_BACKGROUND) 
-		gtk_widget_modify_bg (widget, GTK_STATE_NORMAL, color);
-
-	else if ((type == PANEL_PIXMAP_BACKGROUND) && (pixmap != NULL)) {
-		/* FIXME */
+	/* taken from the Trash Applet */
+	GtkRcStyle *rc_style;
+	GtkStyle *style;
+	
+	/* reset style */
+	gtk_widget_set_style (GTK_WIDGET (applet), NULL);
+	rc_style = gtk_rc_style_new ();
+	gtk_widget_modify_style (GTK_WIDGET (applet), rc_style);
+	g_object_unref (rc_style);
+	
+	switch (type) {
+	case PANEL_COLOR_BACKGROUND:
+		gtk_widget_modify_bg (GTK_WIDGET (applet),
+				      GTK_STATE_NORMAL, 
+				      color);
+		break;
+		
+	case PANEL_PIXMAP_BACKGROUND:
+		style = gtk_style_copy (GTK_WIDGET (applet)->style);
+		if (style->bg_pixmap[GTK_STATE_NORMAL] != NULL)
+			g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
+		style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (pixmap);
+		gtk_widget_set_style (GTK_WIDGET (applet), style);
+		break;
+		
+	case PANEL_NO_BACKGROUND:
+	default:
+		break;
 	}
 }
 
@@ -663,12 +679,18 @@ new_launcher_dialog_response_cb (GtkWidget *dialog,
 		
 	} else if (response == GTK_RESPONSE_OK) { 
 		GnomeDesktopItem *ditem;
-		char             *uri;
+		char             *uri, *path;
+		GnomeVFSURI      *vfs_uri;
 
 		uri = quick_lounge_util__get_unique_uri (quick_lounge);
 
+		vfs_uri = gnome_vfs_uri_new (uri);
+		path = gnome_vfs_uri_to_string (vfs_uri, GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
+		gnome_vfs_uri_unref (vfs_uri);
+
 		ditem = gnome_ditem_edit_get_ditem (dedit);
-		gnome_desktop_item_set_location_file (ditem, uri + strlen ("file://"));
+		gnome_desktop_item_set_location_file (ditem, path);
+		g_free (path);
 
 		if (! gnome_desktop_item_save (ditem, NULL, TRUE, NULL)) {
 			_gtk_error_dialog_run (NULL, 
@@ -830,7 +852,7 @@ populate_context_menu (GtkWidget *popup_menu,
 
 	/**/
 
-	item = gtk_image_menu_item_new_with_mnemonic (_("Add _Launcher"));
+	item = gtk_image_menu_item_new_with_mnemonic (_("Add _New Launcher"));
 	gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), item);
 	gtk_widget_show_all (item);
 
@@ -841,7 +863,7 @@ populate_context_menu (GtkWidget *popup_menu,
 
 	/**/
 
-	item = gtk_image_menu_item_new_with_mnemonic (_("Add From _Menu"));
+	item = gtk_image_menu_item_new_with_mnemonic (_("Add _Existing Launcher"));
 	gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), item);
 	gtk_widget_show_all (item);
 
@@ -952,10 +974,9 @@ drag_data_received  (GtkWidget          *widget,
 		if (! is_desktop_file (src_uri))
 			continue;
 
-		uri = g_strconcat (quick_lounge->location,
-				   "/",
-				   file_name_from_path (src_uri),
-				   NULL);
+		uri = g_build_filename (quick_lounge->location,
+					file_name_from_path (src_uri),
+					NULL);
 
 		if (strcmp (src_uri, uri) == 0) {
 			button = get_button_from_uri (quick_box, uri);
@@ -1078,10 +1099,9 @@ get_order_list (QuickLounge *quick_lounge)
 		if (*filename == 0)
 			uri = g_strdup ("");
 		else
-			uri = g_strconcat (quick_lounge->location, 
-					   "/", 
-					   filename, 
-					   NULL);
+			uri = g_build_filename (quick_lounge->location, 
+						filename, 
+						NULL);
 
 		result = g_list_prepend (result, uri);
 	}
@@ -1263,6 +1283,9 @@ quick_lounge_set_min_visible_cols (QuickLounge *quick_lounge,
 				   int          cols)
 {
 	GError *error = NULL;
+
+	if (cols >= quick_box_get_max_visible_cols (quick_lounge->quick_box))
+		cols = quick_box_get_min_visible_cols (quick_lounge->quick_box);
 	
 	quick_box_set_min_visible_cols (quick_lounge->quick_box, cols);
 	panel_applet_gconf_set_int (PANEL_APPLET (quick_lounge->applet),
@@ -1280,6 +1303,9 @@ quick_lounge_set_max_size (QuickLounge *quick_lounge,
 {
 	GError *error = NULL;
 	
+	if (value <= quick_box_get_min_visible_cols (quick_lounge->quick_box))
+		value = quick_box_get_max_visible_cols (quick_lounge->quick_box);
+
 	quick_box_set_max_visible_cols (quick_lounge->quick_box, value);
 	panel_applet_gconf_set_int (PANEL_APPLET (quick_lounge->applet),
 				    PREFS_MAX_VISIBLE_COLS,
@@ -1313,7 +1339,7 @@ get_unique_dirname (void)
 	char       *base_dir;
 	char       *unique_dir = NULL;
         
-	base_dir = g_strconcat (g_get_home_dir (), "/", BASE_DIR, NULL);
+	base_dir = g_build_filename (g_get_home_dir (), BASE_DIR, NULL);
 	ensure_dir_exists (base_dir, 0700);
 
         while (unique_dir == NULL) {
@@ -1322,10 +1348,7 @@ get_unique_dirname (void)
 		gboolean  exists;
 
 		name = g_strdup_printf ("%d.%d", getpid (), n++);
-		path = g_strconcat (base_dir,
-				    "/",
-				    name,
-				    NULL);
+		path = g_build_filename (base_dir, name, NULL);
 		g_free (name);
 
 		exists = path_is_dir (path);
@@ -1359,10 +1382,14 @@ copy_default_items_to_uri (const char *to_uri)
 		char *source_uri;
 		char *dest_uri;
 
-		source_uri = g_strconcat ("file://", DATADIR, 
-					  "/applications/", default_items[i],
-					  NULL);
-		dest_uri = g_strconcat (to_uri, "/", default_items[i], NULL); 
+		source_uri = g_build_filename ("file://", 
+					       DATADIR, 
+					       "applications", 
+					       default_items[i],
+					       NULL);
+		dest_uri = g_build_filename (to_uri, 
+					     default_items[i], 
+					     NULL); 
 
 		file_copy (source_uri, dest_uri);
 
